@@ -1,4 +1,4 @@
-# Copyright 2013 Jeffrey Kegler
+# Copyright 2014 Jeffrey Kegler
 # This file is part of Marpa::R2.  Marpa::R2 is free software: you can
 # redistribute it and/or modify it under the terms of the GNU Lesser
 # General Public License as published by the Free Software Foundation,
@@ -20,7 +20,7 @@ use strict;
 use warnings;
 
 use vars qw($VERSION $STRING_VERSION @ISA $DEBUG);
-$VERSION        = '2.072000';
+$VERSION        = '2.098000';
 $STRING_VERSION = $VERSION;
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 $VERSION = eval $VERSION;
@@ -29,24 +29,55 @@ $DEBUG = 0;
 
 use Carp;
 use English qw( -no_match_vars );
+use XSLoader;
 
 use Marpa::R2::Version;
 
 $Marpa::R2::USING_XS = 1;
 $Marpa::R2::USING_PP = 0;
+$Marpa::R2::LIBMARPA_FILE = '[built-in]';
+
+LOAD_EXPLICIT_LIBRARY: {
+    last LOAD_EXPLICIT_LIBRARY if  not $ENV{'MARPA_AUTHOR_TEST'};
+    my $file = $ENV{MARPA_LIBRARY};
+    last LOAD_EXPLICIT_LIBRARY if  not $file;
 
-eval {
-    require XSLoader;
-    XSLoader::load( 'Marpa::R2', $Marpa::R2::STRING_VERSION );
-    1;
-} or eval {
     require DynaLoader;
-## no critic(ClassHierarchies::ProhibitExplicitISA)
-    push @ISA, 'DynaLoader';
-    Dynaloader::bootstrap Marpa::R2 $Marpa::R2::STRING_VERSION;
-    1;
-} or Carp::croak("Could not load XS version of Marpa::R2: $EVAL_ERROR");
+    package DynaLoader;
+    my $bs = $file;
+    $bs =~ s/(\.\w+)?(;\d*)?$/\.bs/; # look for .bs 'beside' the library
 
+    if (-s $bs) { # only read file if it's not empty
+#       print STDERR "BS: $bs ($^O, $dlsrc)\n" if $dl_debug;
+        eval { do $bs; };
+        warn "$bs: $@\n" if $@;
+    }
+
+    my $bootname = "marpa_g_new";
+    @DynaLoader::dl_require_symbols = ($bootname);
+
+    my $libref = dl_load_file($file, 0) or do { 
+        require Carp;
+        Carp::croak("Can't load libmarpa library: '$file'" . dl_error());
+    };
+    push(@DynaLoader::dl_librefs,$libref);  # record loaded object
+
+    my @unresolved = dl_undef_symbols();
+    if (@unresolved) {
+        require Carp;
+        Carp::carp("Undefined symbols present after loading $file: @unresolved\n");
+    }
+
+    dl_find_symbol($libref, $bootname) or do {
+        require Carp;
+        Carp::croak("Can't find '$bootname' symbol in $file\n");
+    };
+
+    push(@DynaLoader::dl_shared_objects, $file); # record files loaded
+    $Marpa::R2::LIBMARPA_FILE = $file;
+}
+
+XSLoader::load( 'Marpa::R2', $Marpa::R2::STRING_VERSION );
 
 if ( not $ENV{'MARPA_AUTHOR_TEST'} ) {
     $Marpa::R2::DEBUG = 0;
@@ -97,9 +128,13 @@ require Marpa::R2::MetaG;
 ( $version_result = version_ok($Marpa::R2::MetaG::VERSION) )
     and die 'Marpa::R2::MetaG::VERSION ', $version_result;
 
-require Marpa::R2::Scanless;
-( $version_result = version_ok($Marpa::R2::Scanless::VERSION) )
-    and die 'Marpa::R2::Scanless::VERSION ', $version_result;
+require Marpa::R2::SLG;
+( $version_result = version_ok($Marpa::R2::Scanless::G::VERSION) )
+    and die 'Marpa::R2::Scanless::G::VERSION ', $version_result;
+
+require Marpa::R2::SLR;
+( $version_result = version_ok($Marpa::R2::Scanless::R::VERSION) )
+    and die 'Marpa::R2::Scanless::R::VERSION ', $version_result;
 
 require Marpa::R2::MetaAST;
 ( $version_result = version_ok($Marpa::R2::MetaAST::VERSION) )
@@ -119,11 +154,46 @@ sub Marpa::R2::exception {
     die($exception) if $Marpa::R2::JUST_DIE;
     CALLER: for ( my $i = 0; 1; $i++) {
         my ($package ) = caller($i);
-	last CALLER if not $package;
-	last CALLER if not 'Marpa::R2::' eq substr $package, 0, 11;
-	$Carp::Internal{ $package } = 1;
+        last CALLER if not $package;
+        last CALLER if not 'Marpa::R2::' eq substr $package, 0, 11;
+        $Carp::Internal{ $package } = 1;
     }
     Carp::croak($exception, q{Marpa::R2 exception});
 }
 
+package Marpa::R2::Internal::X;
+
+use overload (
+    q{""} => sub {
+        my ($self) = @_;
+        return $self->{message} // $self->{fallback_message};
+    },
+    fallback => 1
+);
+
+sub new {
+    my ( $class, @hash_ref_args ) = @_;
+    my %x_object = ();
+    for my $hash_ref_arg (@hash_ref_args) {
+        if ( ref $hash_ref_arg ne "HASH" ) {
+            my $ref_type = ref $hash_ref_arg;
+            my $ref_desc = $ref_type ? "ref to $ref_type" : "not a ref";
+            die
+                "Internal error: args to Marpa::R2::Internal::X->new is $ref_desc -- it should be hash ref";
+        } ## end if ( ref $hash_ref_arg ne "HASH" )
+        $x_object{$_} = $hash_ref_arg->{$_} for keys %{$hash_ref_arg};
+    } ## end for my $hash_ref_arg (@hash_ref_args)
+    my $name = $x_object{name};
+    die("Internal error: an excepion must have a name") if not $name;
+    $x_object{fallback_message} = qq{Exception "$name" thrown};
+    return bless \%x_object, $class;
+} ## end sub new
+
+sub name {
+    my ($self) = @_;
+    return $self->{name};
+}
+
 1;
+
+# vim: set expandtab shiftwidth=4:
